/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#include "boat.h"
#include "bomb.h"
#include "pixmapitem.h"
#include "graphicsscene.h"

static const int MAX_BOMB = 5;

Boat::Boat(QGraphicsItem * parent, Qt::WindowFlags wFlags)
    : QGraphicsWidget(parent,wFlags), currentSpeed(0), currentDirection(Boat::None), movementAnimation(0),isAboutToDie(false)
{
    pixmapItem = new PixmapItem(QString("boat"),GraphicsScene::Big, this);
    setZValue(4);
    setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    setFlags(QGraphicsItem::ItemIsMovable |  QGraphicsItem::ItemIsFocusable);
    resize(pixmapItem->boundingRect().size());
}

void Boat::keyPressEvent(QKeyEvent * event)
{
    if (isAboutToDie)
        return;
    switch(event->key()) {
      case Qt::Key_Left :
          if (currentDirection == Boat::None && currentSpeed == 0)
              currentDirection = Boat::Left;
              if (currentDirection == Boat::Left && currentSpeed < 3) {
                      currentSpeed++;
                      updateBoatMovement();
              }
              else if (currentDirection == Boat::Right && currentSpeed != 0) {
                      currentSpeed--;
                      updateBoatMovement();
              }
              if (currentSpeed == 0)
                      currentDirection = Boat::None;
              break;

      case Qt::Key_Right :
            if (currentDirection == Boat::None && currentSpeed == 0)
                currentDirection = Boat::Right;
            if (currentDirection == Boat::Right && currentSpeed < 3) {
                currentSpeed++;
                updateBoatMovement();
            }
            else if (currentDirection == Boat::Left && currentSpeed != 0) {
                currentSpeed--;
                updateBoatMovement();
            }
            if (currentSpeed == 0)
                currentDirection = Boat::None;
            break;

      case Qt::Key_Down :
            if (bombs.size() <= MAX_BOMB && !isAboutToDie) {
                Bomb *b = new Bomb();
                b->setPos(x() - b->size().width(), y());
                scene()->addItem(b);
                b->launch(Bomb::Left);
                bombs.insert(b);
                connect(b, SIGNAL(destroyed(QObject *)),this,SLOT(onBombDestroyed(QObject *)));
            }
            QGraphicsWidget::keyPressEvent(event);
            break;

      case Qt::Key_Up :
            if (bombs.size() <= MAX_BOMB && !isAboutToDie) {
                Bomb *b = new Bomb();
                b->setPos(x()+size().width(), y());
                scene()->addItem(b);
                b->launch(Bomb::Right);
                bombs.insert(b);
                connect(b, SIGNAL(destroyed(QObject *)),this,SLOT(onBombDestroyed(QObject *)));
            }
            QGraphicsWidget::keyPressEvent(event);
            break;

      default : QGraphicsWidget::keyPressEvent(event);break;
    }
    event->accept();
}

void Boat::updateBoatMovement()
{
    if (isAboutToDie)
        return;
    if (movementAnimation) {
        movementAnimation->stop();
        delete movementAnimation;
        movementAnimation = 0;
    }
    if (currentSpeed == 0 || currentDirection == Boat::None)
        return;

    movementAnimation = new QtPropertyAnimation(this, "pos");
    if (currentDirection == Boat::Left) {
        movementAnimation->setEndValue(QPointF(0,y()));
        movementAnimation->setDuration(x()/currentSpeed*15);
    }
    else /*if (currentDirection == Boat::Right)*/ {
        movementAnimation->setEndValue(QPointF(scene()->width()-size().width(),y()));
        movementAnimation->setDuration((scene()->width()-size().width()-x())/currentSpeed*15);
    }
    movementAnimation->start();
    connect(movementAnimation,SIGNAL(finished()),this,SLOT(onAnimationMovementFinished()));
}

void Boat::onAnimationMovementFinished()
{
     currentDirection = Boat::None;
     currentSpeed = 0;
}

void Boat::destroy()
{
    isAboutToDie = true;
    if (movementAnimation) {
        movementAnimation->stop();
        movementAnimation->deleteLater();
        movementAnimation = 0;
    }
    foreach (Bomb *bomb,bombs) {
        bomb->destroy();
    }
    deleteLater();

}

void Boat::onBombDestroyed(QObject * obj)
{
    bombs.remove(static_cast<Bomb *>(obj));
}

int Boat::type() const
{
    return Type;
}
