/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/


#include "graphicsscene.h"
#include "boat.h"
#include "submarine.h"
#include "torpedo.h"
#include "pixmapitem.h"

GraphicsScene::GraphicsScene(int x, int y, int width, int height, Mode mode)
    : QGraphicsScene(x,y,width,height), currentLevel(1), gameIsRunning(false), mode(mode)
{
    backgroundItem = new PixmapItem(QString("background"),mode);
    backgroundItem->setZValue(1);
    backgroundItem->setPos(0,0);
    addItem(backgroundItem);

    PixmapItem *surfaceItem = new PixmapItem(QString("surface"),mode);
    surfaceItem->setZValue(3);
    surfaceItem->setPos(0,sealLevel() - surfaceItem->boundingRect().height()/2);
    addItem(surfaceItem);
}

GraphicsScene::~GraphicsScene()
{
}

bool GraphicsScene::gameInProgress() const
{
     return gameIsRunning;
}

float GraphicsScene::sealLevel() const
{
     if (mode == Big)
        return 220;
     else
        return 160;
}

void GraphicsScene::initializeLevel()
{
    boat = new Boat();
    addItem(boat);
    setFocusItem(boat,Qt::OtherFocusReason);
    boat->setPos(width()/2, sealLevel() - boat->size().height());
    connect(boat, SIGNAL(destroyed(QObject *)), this, SLOT(onBoatDestroy(QObject *)));
    gameIsRunning = true;

    for (int i = 0 ; i <= 5 ; ++i)
        subMarines.insert(new SubMarine());

    foreach (SubMarine *sub,subMarines) {
        addItem(sub);
        connect(sub,SIGNAL(destroyed(QObject *)), this, SLOT(onSubMarineDestroy(QObject *)));
        int random = (qrand() % 15 + 1);
        qreal x = random == 13 || random == 5 ? 0 : width() - sub->size().width();
        qreal y = height() -(qrand() % 150 + 1) - sub->size().height();
        sub->setPos(x,y);
        sub->setCurrentDirection(x == 0 ? SubMarine::Right : SubMarine::Left);
        sub->setCurrentSpeed(qrand() % 3 + 1);
    }
}

void GraphicsScene::pause()
{
    //### TODO
}

void GraphicsScene::mousePressEvent (QGraphicsSceneMouseEvent * event)
{
    event->ignore();
}

void GraphicsScene::onSubMarineDestroy(QObject * obj)
{
    subMarines.remove(static_cast<SubMarine *>(obj));
    if (subMarines.size() == 0 && gameIsRunning) {
        qDebug()<<"You win!!!!!!!!!!!!";
        gameIsRunning = false;
        if (boat) {
            disconnect(boat, SIGNAL(destroyed(QObject *)), this, SLOT(onBoatDestroy(QObject *)));
            boat->destroy();
        }
        foreach (QGraphicsItem *item, items()) {
                if (Torpedo *torp = dynamic_cast<Torpedo *>(item))
                    torp->destroy();
        }
    }
}

void GraphicsScene::onBoatDestroy(QObject *)
{
    qDebug()<<"You Loose :( :(";
    gameIsRunning = false;
    foreach (SubMarine *sub,subMarines) {
        sub->destroy();
    }
}
