/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#ifndef __SUBMARINE__H__
#define __SUBMARINE__H__

#include <QtGui>
#if defined(QT_EXPERIMENTAL_SOLUTION)
#include "qanimation.h"
#endif

class PixmapItem;

class Torpedo;

class SubMarine : public QGraphicsWidget
{
Q_OBJECT
Q_PROPERTY(QPointF pos READ pos WRITE setPos)
Q_PROPERTY(qreal rotation READ rotation WRITE setRotation)
public:
    enum Movement {
       None = 0,
       Left,
       Right     
    };
    enum { Type = UserType + 1 };
    SubMarine(QGraphicsItem * parent = 0, Qt::WindowFlags wFlags = 0);
    ~SubMarine();
    void setCurrentDirection(Movement direction);
    void setCurrentSpeed(int speed);
    void launchTorpedo(int speed);
    void explose();
    void destroy();
    double rotation() const;
    void setRotation(double rr);

    virtual int type() const;


private slots:
    void onAnimationMovementFinished();
    void onAnimationMovementValueChanged(const QVariant &);
    void onAnimationRotationFinished();
    void onTorpedoDestroyed(QObject * obj);

private:
    void updateSubMarineMovement();

    int currentSpeed;
    double currentRotation;
    bool rotateLeft;
    Movement currentDirection;
    QtAnimation *movementAnimation;
    QtAnimation *returnAnimation;
    bool isAboutToDie;
    PixmapItem *pixmapItem;
    QSet<Torpedo *> torpedos;
};

#endif //__SUBMARINE__H__
