/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#ifndef QANIMATION_H
#define QANIMATION_H

#if defined(QT_EXPERIMENTAL_SOLUTION)
# include "qabstractanimation.h"
# include "qeasingcurve.h"
#else
# include <QtCore/qeasingcurve.h>
# include <QtCore/qabstractanimation.h>
#endif
#include <QtCore/qlist.h>
#include <QtCore/qpoint.h>
#include <QtCore/qvariant.h>
#include <QtCore/qpair.h>

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Core)

class QtAnimationPrivate;
class Q_ANIMATION_EXPORT QtAnimation : public QtAbstractAnimation
{
    Q_OBJECT
    Q_PROPERTY(QVariant startValue READ startValue WRITE setStartValue) /*NOTIFY startValueChanged*/
    Q_PROPERTY(QVariant endValue READ endValue WRITE setEndValue) /*NOTIFY endValueChanged*/
    Q_PROPERTY(QVariant currentValue READ currentValue NOTIFY valueChanged)
    Q_PROPERTY(int duration READ duration WRITE setDuration NOTIFY durationChanged)
    Q_PROPERTY(QtEasingCurve easingCurve READ easingCurve WRITE setEasingCurve) /*NOTIFY easingCurveChanged*/

public:
    typedef QPair<qreal, QVariant> KeyFrame;
    typedef QVector<KeyFrame> KeyFrames;

    QtAnimation(QObject *parent = 0);

    QVariant startValue() const;
    void setStartValue(const QVariant &value);

    QVariant endValue() const;
    void setEndValue(const QVariant &value);

    QVariant valueAt(qreal step) const;
    void setValueAt(qreal step, const QVariant &value);

    KeyFrames keyFrames() const;
    void setKeyFrames(const KeyFrames &values);

    QVariant currentValue() const;

    int duration() const;
    void setDuration(int msecs);

    QtEasingCurve easingCurve() const;
    void setEasingCurve(const QtEasingCurve &easing);

    typedef QVariant (*Interpolator)(const QVariant &from, const QVariant &to, qreal progress);
    static void registerInterpolator(Interpolator func, int interpolationType);

Q_SIGNALS:
    void valueChanged(const QVariant &value);
    void durationChanged(int msecs);

protected:
    QtAnimation(QtAnimationPrivate &dd, QObject *parent = 0);
    bool event(QEvent *event);

    QVariant defaultStartValue() const;
    void setDefaultStartValue(const QVariant &);

    void updateCurrentTime(int msecs);
    void updateState(QtAbstractAnimation::State oldState, QtAbstractAnimation::State newState);

    virtual void updateCurrentValue(const QVariant &value) = 0;
    virtual QVariant interpolated(const QVariant &from, const QVariant &to, qreal progress) const;

private:
    Q_DISABLE_COPY(QtAnimation)
    Q_DECLARE_PRIVATE(QtAnimation)
};

template <typename T>
static void qRegisterAnimationInterpolator(QtAnimation::Interpolator func
#ifndef qdoc
, T * /* dummy */ = 0
#endif
) {
    QtAnimation::registerInterpolator(func, qMetaTypeId<T>());
}


QT_END_NAMESPACE

#endif //QANIMATION_H
