/****************************************************************************
**
** This file is part of a Qt Solutions component.
** 
** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
** 
** Contact:  Qt Software Information (qt-info@nokia.com)
** 
** Commercial Usage  
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Solutions Commercial License Agreement provided
** with the Software or, alternatively, in accordance with the terms
** contained in a written agreement between you and Nokia.
** 
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
** 
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
** 
** GNU General Public License Usage 
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** 
** Please note Third Party Software included with Qt Solutions may impose
** additional restrictions and it is the user's responsibility to ensure
** that they have met the licensing requirements of the GPL, LGPL, or Qt
** Solutions Commercial license and the relevant license of the Third
** Party Software they are using.
** 
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** 
****************************************************************************/



#ifndef QSEQUENTIALANIMATIONGROUP_H
#define QSEQUENTIALANIMATIONGROUP_H

#if defined(QT_EXPERIMENTAL_SOLUTION)
# include "qanimationgroup.h"
#else
# include <QtCore/qanimationgroup.h>
#endif

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Core)

class QtPauseAnimation;
class QtSequentialAnimationGroupPrivate;

class Q_ANIMATION_EXPORT QtSequentialAnimationGroup : public QtAnimationGroup
{
    Q_OBJECT

public:
    QtSequentialAnimationGroup(QObject *parent = 0);
    ~QtSequentialAnimationGroup();

    QtPauseAnimation *addPause(int msecs);
    void insertAt(int index, QtAbstractAnimation *animation);
    QtPauseAnimation *insertPauseAt(int index, int msecs);

    QtAbstractAnimation *currentAnimation() const;
    int duration() const;

protected:
    QtSequentialAnimationGroup(QtSequentialAnimationGroupPrivate &dd, QObject *parent);

    void updateCurrentTime(int msecs);
    void updateState(QtAbstractAnimation::State oldState, QtAbstractAnimation::State newState);
    void updateDirection(QtAbstractAnimation::Direction direction);
    void animationInsertedAt(int index);
    void animationRemovedAt(int index);

private:
    Q_DISABLE_COPY(QtSequentialAnimationGroup)
    Q_DECLARE_PRIVATE(QtSequentialAnimationGroup)
    Q_PRIVATE_SLOT(d_func(), void _q_uncontrolledAnimationFinished())
};

QT_END_NAMESPACE

QT_END_HEADER

#endif //QSEQUENTIALANIMATIONGROUP_H
